-----------------------
-------VARIABLES-------
-----------------------


local ADDON_NAME, namespace = ...;
local functions = namespace.functions;
local variables = namespace.variables;

local llr_EventFrame = CreateFrame("Frame");
local currItem = "";

--Clean up the trade time remaining string so that it can be used cross locales.
--As seen in the RCLootConcil2 addon (https://github.com/evil-morfar/RCLootCouncil2).
local trimmedString = escapePatternSymbols(BIND_TRADE_TIME_REMAINING):gsub("%%%%s", "%(%.%+%)"); 

--Create frame to hold tooltip for scanning.
CreateFrame( "GameTooltip", "LPTLootRoll_ToolTip", nil, "GameTooltipTemplate");
LPTLootRoll_ToolTip:UnregisterAllEvents();


-----------------------
----LOCAL FUNCTIONS----
-----------------------


--Function for checking if it is mog that the player does not know and can collect.
local function usableMog(link)
	local appearId, source = C_TransmogCollection.GetItemInfo(link);
	local collectable = select(2, C_TransmogCollection.PlayerCanCollectSource(source));
	local sources = C_TransmogCollection.GetAppearanceSources(appearId);
	
	if sources and collectable then
		for i, v in pairs(sources) do
			if v.isCollected then 
				return false;
			end
		end
	end
	
	return collectable;
end

--Get the main stats of the item.
local function getMainStats(link)
	local stats = {
		agility = false,
		strength = false,
		intellect = false
	};
	local itemStats = GetItemStats(link);

	stats.agility 	= tostring(itemStats["ITEM_MOD_AGILITY_SHORT"]or 0)~="0";
	stats.strength 	= tostring(itemStats["ITEM_MOD_STRENGTH_SHORT"]or 0)~="0";
	stats.intellect = tostring(itemStats["ITEM_MOD_INTELLECT_SHORT"]or 0)~="0";

	return stats;
end

--Function for checking if stats are correct.
local function usableStats(link)
	local stats = getMainStats(link);
	local agilityUsage   = stats.agility   and variables.classArray[variables.playerClassId].stats.agility;
	local strengthUsage  = stats.strength  and variables.classArray[variables.playerClassId].stats.strength;
	local intellectUsage = stats.intellect and variables.classArray[variables.playerClassId].stats.intellect;

	return agilityUsage or strengthUsage or intellectUsage;
end


-----------------------
-----EVENT HANDLING----
-----------------------


llr_EventFrame:RegisterEvent("CHAT_MSG_RAID_WARNING");
llr_EventFrame:RegisterEvent('GROUP_ROSTER_UPDATE');

llr_EventFrame:SetScript("OnEvent", 
	function(self,event,...)
		local arg1, arg2, arg3, arg4, arg5 = ...;
		if event == "CHAT_MSG_RAID_WARNING" then
			local name, _ = strsplit("-", arg2);
			
			--Check if the person sending a raidwarning is the raid leader or if assist setting is enabled.
			if UnitIsGroupLeader(name) or llrSettings.assistMode then
				--Hide user window as new item is being rolled.
				functions.hideUser();
				
				local _, link, _, _, _, _, _, _, itemEquipLoc, _, _, itemClassID, itemSubClassID = GetItemInfo(arg1);
				local isPet = itemClassID == 15 and itemSubClassID == 2;

				if 
					link and 
					(
						itemClassID == 4 or --Armor
						itemClassID == 2 or --Weapon
						isPet 				--Pet
					)
				then
					local usabilityVar = 
					--Check if it is a pet.
					isPet or
					--Check if it item is a cloak.
					itemEquipLoc == "INVTYPE_CLOAK" or 
					--Check if it is a ring/neck/trinket, if so mark it as usable.
					(itemClassID == 4 and itemSubClassID == 0 and itemEquipLoc ~= "INVTYPE_HOLDABLE") or
					--Check if collectable mog.
					usableMog(link) or 
					(	
						(
							--Check if item is an off-hand.
							itemEquipLoc == "INVTYPE_HOLDABLE" or
							--Check if item is an equippable weapon/armor piece.
							variables.classArray[variables.playerClassId][itemClassID][itemSubClassID]
						) and 
						--If it is either of these, check if stats are usable for the player's class.
						usableStats(link)
					);

					if UnitIsGroupLeader("player") or llrSettings.leadMode then
						functions.historyLengthControl();

						--Insert the new item into the rollhistory array.
						tinsert(variables.rollHistory, 1, 
							{
								item 		= link,
								mainList 	= {},
								offList 	= {},
								mogList 	= {},
								otherList   = {},
								distributed = false
							}
						);

						functions.leaderEvent(usabilityVar);
					elseif usabilityVar then
						functions.userEvent(link);
					end
				end 
			end
		elseif event == "CHAT_MSG_LOOT" then
			local name = strsplit("-", arg2);

			if name == variables.playerName then
				currItem = select(2, GetItemInfo(arg1));
			end
		elseif event == "BAG_UPDATE" then
			if currItem == "" then
				return;
			end

			local bag = arg1;

			for slot = 1, GetContainerNumSlots(bag) do
				local bagLink = GetContainerItemLink(bag, slot);

				if bagLink and bagLink == currItem then
					LPTLootRoll_ToolTip:SetOwner(UIParent, "ANCHOR_NONE");
					LPTLootRoll_ToolTip:SetBagItem(bag, slot);

					for i = 1, LPTLootRoll_ToolTip:NumLines() or 0 do 
						local left = getglobal(LPTLootRoll_ToolTip:GetName() .. "TextLeft" .. i);
						local leftText = left:GetText() or "";

						if leftText:match(trimmedString) then
							LPTLootRoll_ToolTip:Hide();

							local itemName, link, _, _, _, _, _, _, _, itemIcon = GetItemInfo(bagLink);
							local data = {["name"] = itemName, ["link"] = link, ["texture"] = itemIcon};
							local dialog = StaticPopup_Show("LPTLootRoll_SendTradeable", link, _, data);

							if dialog then
								dialog.data = link;
							end

							currItem = "";
							return;
						end
					end
				end
			end

			currItem = "";
		elseif event == "GROUP_ROSTER_UPDATE" then
			functions.toggleLootListener();
			functions.toggleWhisperListener();
			functions.controlRollListener();
		end
	end
);


-----------------------
----GLOBAL FUNCTIONS---
-----------------------


--Function for toggeling the tradeable item listener.
function functions.toggleLootListener()
	if llrSettings.lootListener and UnitInRaid("player") then
		llr_EventFrame:RegisterEvent("CHAT_MSG_LOOT");
		llr_EventFrame:RegisterEvent("BAG_UPDATE");
	else 
		llr_EventFrame:UnregisterEvent("CHAT_MSG_LOOT");
		llr_EventFrame:UnregisterEvent("BAG_UPDATE");
	end
end